#include <avr/interrupt.h>
#include <stdbool.h>

#include "adc.h"

volatile int16_t signal = 0;	// filter result (signal power)
volatile bool new_value = 0;

volatile int16_t v0 = 0;		// goertzel filter variables
volatile int16_t v1 = 0;
volatile int16_t v2 = 0;
volatile int16_t filtidx = 0;	// current filter step

#define N 128					// goertzel filter length; 
								// frequency resolution B = Fs/N
#define A 106					// filter coefficient: 2*cos(4761Hz*2*pi/Fs)*128
                				// scaled with 128

/* ADC initialisieren */
extern void ADC_Init(void)
{
	ADMUX = (1<<REFS1) | (1<<REFS0);	// AVcc als Referenz, ADC0
	ADCSRA = (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);			// Frequenzvorteiler=128
	ADCSRA |= (1<<ADEN);									// ADC aktivieren
	 /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
	also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
 
	ADCSRA |= (1<<ADSC);					// eine ADC-Wandlung 
	while (ADCSRA & (1<<ADSC) ) {}			// auf Abschluss der Konvertierung warten
	/* ADCW muss einmal gelesen werden, sonst wird Ergebnis der nchsten
	Wandlung nicht bernommen. */
	signal = ADCW;
	signal = 0;

	ADCSRA |= (1<<ADFR) | (1<<ADIE);		// Free Running Mode, Interrupt Enable
	ADCSRA |= (1<<ADSC);					// Start Conversion
}

// goertzel algorithm in ADC conversion complete ISR
ISR(ADC_vect)
{
  uint16_t adcval;

  //read 16bit ADC value
  adcval = ADCW;

  //goertzel
  if (filtidx == N)				// outer filter
  {
    filtidx = 0;

    v1 = v1/256;				// scaling with 128
    v2 = v2/256;
    signal = (v1*v2*A)/512;		// scaling with 256
    signal = v1*v1 + v2*v2 - signal;
	new_value = true;
    v1 = 0;
    v2 = 0;

  }
  else                  		// inner filter
  {
    v0 = A*v1/128;     			// scaling with 128
    v0 = adcval + v0 - v2;
    v2 = v1;
    v1 = v0;
    filtidx++;
  }
}
